@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -134,6 +134,24 @@ xps_add_fixed_page(xps_context_t *ctx, char *name, int width, int height)
         if (!strcmp(page->name, name))
             return;
 
+    if (ctx->page_range && !ctx->page_range->page_list)
+    {
+        ctx->page_range->current++;
+
+        if (ctx->page_range->reverse)
+        {
+            if (ctx->page_range->current < ctx->page_range->last ||
+                ctx->page_range->current > ctx->page_range->first)
+                return;
+        }
+        else
+        {
+            if ((ctx->page_range->first != 0 && ctx->page_range->current < ctx->page_range->first) ||
+                (ctx->page_range->last != 0 && ctx->page_range->current > ctx->page_range->last))
+                return;
+        }
+    }
+
     if_debug1m('|', ctx->memory, "doc: adding page %s\n", name);
 
     page = xps_alloc(ctx, sizeof(xps_page_t));
@@ -153,8 +171,17 @@ xps_add_fixed_page(xps_context_t *ctx, char *name, int width, int height)
     }
     else
     {
-        ctx->last_page->next = page;
-        ctx->last_page = page;
+        /* FirstPage < LastPage? */
+        if (ctx->page_range && ctx->page_range->reverse)
+        {
+            page->next = ctx->first_page;
+            ctx->first_page = page;
+        }
+        else
+        {
+            ctx->last_page->next = page;
+            ctx->last_page = page;
+        }
     }
 }
 
@@ -265,7 +292,7 @@ xps_parse_metadata(xps_context_t *ctx, xps_part_t *part)
     char *s;
 
     /* Save directory name part */
-    xps_strlcpy(buf, part->name, sizeof buf);
+    gs_strlcpy(buf, part->name, sizeof buf);
     s = strrchr(buf, '/');
     if (s)
         s[0] = 0;
